==> justai-3.0.0-py3-none-any.whl: justai/models/openai_models.py <==
import json
import os

import openai
import tiktoken
from dotenv import load_dotenv
from openai import APIConnectionError, APIError, RateLimitError, OpenAI
from openai._types import NOT_GIVEN

from justai.tools.cache import cached
from justai.tools.display import color_print, ERROR_COLOR, DEBUG_COLOR1, DEBUG_COLOR2, SYSTEM_COLOR
from justai.models.model import Model


class OpenAIModel(Model):
    def __init__(self, model_name: str, params: dict = None):
        self.model_name = model_name
        self.system_message = f"You are {model_name}, a large language model trained by OpenAI."

        # Authentication
        if not os.getenv("OPENAI_API_KEY"):
            load_dotenv()  # Load the .env file into the environment
        openai.api_key = os.getenv("OPENAI_API_KEY")
        if not openai.api_key:
            color_print("No OpenAI API key found. Create one at https://platform.openai.com/account/api-keys and " +
                        "set it in the .env file like OPENAI_API_KEY=here_comes_your_key.", color=ERROR_COLOR)
        self.client = OpenAI()
        self.debug = params.get('debug', False)

        # The maximum number of tokens to generate in the completion.
        # Defaults to 16
        # The token count of your prompt plus max_tokens cannot exceed the model's context length.
        # Most models have a context length of 2048 tokens (except for the newest models, which support 4096).
        self.max_tokens = params.get('max_tokens', 800)

        # What sampling temperature to use, between 0 and 2.
        # Higher values like 0.8 will make the output more random, while lower values like 0.2
        # will make it more focused and deterministic.
        # We generally recommend altering this or top_p but not both
        # Defaults to 1
        self.temperature = params.get('temperature', 0.5)

        # An alternative to sampling with temperature, called nucleus sampling,
        # where the model considers the results of the tokens with top_p probability mass.
        # So 0.1 means only the tokens comprising the top 10% probability mass are considered.
        # We generally recommend altering this or temperature but not both.
        # Defaults to 1
        self.top_p = params.get('top_p', 1)

        # How many completions to generate for each prompt.
        # Because this parameter generates many completions, it can quickly consume your token quota.
        # Use carefully and ensure that you have reasonable settings for max_tokens and stop.
        self.n = params.get('n', 1)

        # Up to 4 sequences where the API will stop generating further tokens.
        # The returned text will not contain the stop sequence.
        # Example: [" Human:", " AI:"]
        self.stop = params.get('stop')

        # Number between -2.0 and 2.0.
        # Positive values penalize new tokens based on whether they appear in the text so far,
        # increasing the model's likelihood to talk about new topics.
        # Defaults to 0
        self.presence_penalty = params.get('presence_penalty', 0)

        # Number between -2.0 and 2.0.
        # Positive values penalize new tokens based on their existing frequency in the text so far,
        # decreasing the model's likelihood to repeat the same line verbatim.
        # Defaults to 0
        self.frequency_penalty = params.get('frequency_penalty', 0)

    def chat(self, prompt: str, return_json: bool, messages: list[dict], use_cache: bool = False, max_retries: int = 3):

        @cached
        def cached_request():
            return chat_completion_request()

        def chat_completion_request():
            if self.debug:
                color_print("\nRunning completion with these messages", color=DEBUG_COLOR1)
                [color_print(m, color=DEBUG_COLOR1) for m in messages if hasattr(m, 'text')]
                print()

            last_error = None
            for _ in range(max_retries):
                try:
                    completion = self.client.chat.completions.create(
                        model=self.model_name,
                        messages=messages,
                        temperature=self.temperature,
                        max_tokens=self.max_tokens,
                        n=self.n,
                        top_p=self.top_p,
                        frequency_penalty=self.frequency_penalty,
                        presence_penalty=self.presence_penalty,
                        stop=self.stop,
                        response_format={"type": "json_object"} if return_json else NOT_GIVEN,
                    )
                    if self.debug and hasattr(completion.choices[0], 'text'):
                        color_print(f"{completion.choices[0].text}", color=DEBUG_COLOR2)
                    return completion
                except APIConnectionError as e:
                    color_print("Connection error.", color=SYSTEM_COLOR)
                    last_error = e
                except APIError as e:
                    color_print("API error", color=SYSTEM_COLOR)
                    last_error = e
                except RateLimitError as e:
                    color_print(f"{self.model_name} is overloaded", color=SYSTEM_COLOR)
                    last_error = e
            print('Too many errors. Aborting.')
            raise last_error

        completion = cached_request() if use_cache else chat_completion_request()

        message_text = completion.choices[0].message.content
        if message_text.startswith('```json'):
            message_text = message_text[7:-3]
        input_token_count = completion.usage.prompt_tokens
        output_token_count = completion.usage.completion_tokens

        text = json.loads(message_text) if return_json else message_text
        return text, input_token_count, output_token_count

    def token_count(self, text: str) -> int:
        encoding = tiktoken.encoding_for_model(self.model_name)
        return len(encoding.encode(text))

==> justai-3.0.1-py3-none-any.whl: justai/models/openai_models.py <==
import json
import os

import openai
import tiktoken
from dotenv import load_dotenv
from openai import APIConnectionError, APIError, RateLimitError, OpenAI
from openai._types import NOT_GIVEN

from justai.tools.cache import cached
from justai.tools.display import color_print, ERROR_COLOR, DEBUG_COLOR1, DEBUG_COLOR2, SYSTEM_COLOR
from justai.models.model import Model


class OpenAIModel(Model):
    def __init__(self, model_name: str, params: dict = None):
        self.model_name = model_name
        self.system_message = f"You are {model_name}, a large language model trained by OpenAI."

        # Authentication
        if not os.getenv("OPENAI_API_KEY"):
            load_dotenv()  # Load the .env file into the environment
        openai.api_key = os.getenv("OPENAI_API_KEY")
        if not openai.api_key:
            color_print("No OpenAI API key found. Create one at https://platform.openai.com/account/api-keys and " +
                        "set it in the .env file like OPENAI_API_KEY=here_comes_your_key.", color=ERROR_COLOR)
        self.client = OpenAI()
        self.debug = params.get('debug', False)

        # The maximum number of tokens to generate in the completion.
        # Defaults to 16
        # The token count of your prompt plus max_tokens cannot exceed the model's context length.
        # Most models have a context length of 2048 tokens (except for the newest models, which support 4096).
        self.max_tokens = params.get('max_tokens', 800)

        # What sampling temperature to use, between 0 and 2.
        # Higher values like 0.8 will make the output more random, while lower values like 0.2
        # will make it more focused and deterministic.
        # We generally recommend altering this or top_p but not both
        # Defaults to 1
        self.temperature = params.get('temperature', 0.5)

        # An alternative to sampling with temperature, called nucleus sampling,
        # where the model considers the results of the tokens with top_p probability mass.
        # So 0.1 means only the tokens comprising the top 10% probability mass are considered.
        # We generally recommend altering this or temperature but not both.
        # Defaults to 1
        self.top_p = params.get('top_p', 1)

        # How many completions to generate for each prompt.
        # Because this parameter generates many completions, it can quickly consume your token quota.
        # Use carefully and ensure that you have reasonable settings for max_tokens and stop.
        self.n = params.get('n', 1)

        # Up to 4 sequences where the API will stop generating further tokens.
        # The returned text will not contain the stop sequence.
        # Example: [" Human:", " AI:"]
        self.stop = params.get('stop')

        # Number between -2.0 and 2.0.
        # Positive values penalize new tokens based on whether they appear in the text so far,
        # increasing the model's likelihood to talk about new topics.
        # Defaults to 0
        self.presence_penalty = params.get('presence_penalty', 0)

        # Number between -2.0 and 2.0.
        # Positive values penalize new tokens based on their existing frequency in the text so far,
        # decreasing the model's likelihood to repeat the same line verbatim.
        # Defaults to 0
        self.frequency_penalty = params.get('frequency_penalty', 0)

    def chat(self, prompt: str, return_json: bool, messages: list[dict], use_cache: bool = False, max_retries: int = 3):

        @cached
        def cached_request(model_name, messages, temperature, max_tokens, n, top_p, frequency_penalty,
                           presence_penalty, stop, return_json):
            return chat_completion_request(model_name, messages, temperature, max_tokens, n, top_p, frequency_penalty,
                                           presence_penalty, stop, return_json)

        def chat_completion_request(model_name, messages, temperature, max_tokens, n, top_p, frequency_penalty,
                                    presence_penalty, stop, return_json):
            completion = self.client.chat.completions.create(
                model=model_name,
                messages=messages,
                temperature=temperature,
                max_tokens=max_tokens,
                n=n,
                top_p=top_p,
                frequency_penalty=frequency_penalty,
                presence_penalty=presence_penalty,
                stop=stop,
                response_format={"type": "json_object"} if return_json else NOT_GIVEN,
            )
            return completion

        if self.debug:
            color_print("\nRunning completion with these messages", color=DEBUG_COLOR1)
            [color_print(m, color=DEBUG_COLOR1) for m in messages if hasattr(m, 'text')]
            print()

        last_error = None
        completion_function = cached_request if use_cache else chat_completion_request
        for _ in range(max_retries):
            try:
                completion = completion_function(self.model_name, messages, self.temperature, self.max_tokens, self.n, 
                                                 self.top_p,  self.frequency_penalty, self.presence_penalty, self.stop, 
                                                 return_json)
                if self.debug and hasattr(completion.choices[0], 'text'):
                    color_print(f"{completion.choices[0].text}", color=DEBUG_COLOR2)
                break
            except APIConnectionError as e:
                color_print("Connection error.", color=SYSTEM_COLOR)
                last_error = e
            except APIError as e:
                color_print("API error", color=SYSTEM_COLOR)
                last_error = e
            except RateLimitError as e:
                color_print(f"{self.model_name} is overloaded", color=SYSTEM_COLOR)
                last_error = e
        else:
            print('Too many errors. Aborting.')
            raise last_error

        message_text = completion.choices[0].message.content
        if message_text.startswith('```json'):
            message_text = message_text[7:-3]
        input_token_count = completion.usage.prompt_tokens
        output_token_count = completion.usage.completion_tokens

        text = json.loads(message_text) if return_json else message_text
        return text, input_token_count, output_token_count

    def token_count(self, text: str) -> int:
        encoding = tiktoken.encoding_for_model(self.model_name)
        return len(encoding.encode(text))
